/* Copyright (c) 2008, Tim Post <tinkertim@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the original program's authors nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* NOTE:
 * This is a bit of an ugly hack, working around the absence of fstat / etc.
 * As more stuff is completed and exposed in libc, this will improve */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "errors.h"
#include "config.h"
#include "util.h"
#include "entry.h"
#include "ls.h"
#include "cmds.h"

static char *cmdname = "ls";

unsigned int ls_scope(const char *path)
{
	int fd;
	DIR *dirp;

	dirp = opendir(path);
	if (dirp) {
		closedir(dirp);
		return LS_DIR;
	}

	fd = open(path, O_RDONLY);
	if (fd > 0) {
		close(fd);
		return LS_FILE;
	}

	return LS_BOGUS;
}

void ls_print(const char *f)
{
	unsigned int scope;

	scope = ls_scope(f);

	/* If this function is called in a readdir loop, LS_BOGUS
	 * can be treated as LS_FILE, since we aren't presenting
	 * the full path to open(), just pointer->d_name as obtained
	 * from opendir(). This will not happen if ../path/to/file
	 * is passed, as may be the argument to ls */

	switch (scope) {
	case LS_DIR:
		printf("%-40s <DIR>\n", f);
		break;
	case LS_FILE:
		printf("%-40s\n", f);
		break;
	/* This is never reached unless in a readdir() loop */
	case LS_BOGUS:
		printf("%-40s\n", f);
		break;
	}
	return;
}

/* Dispays help for ls in various levels */
void * help_cmd_ls(unsigned int level)
{
	if (level == HELP_SHORT) {
		printf("`%s' lists files and directories.\n", cmdname);
	} else {
		help_cmd_ls(HELP_SHORT);
		printf("  `%s' [path], if no path is given the current "
				"working directory is used.\n", cmdname);
	}

	return CMD_VOID;
}

int * cmd_ls(char **argv)
{
	unsigned int argc;
	unsigned int scope;
	char *buff;
	char s[PATH_MAX];

	DIR *dirp;
	struct dirent *dp;

	/* Count the arguments */
	for (argc = 0; argv[argc] != NULL; argc ++);

	if (argc > 2) {
		printf("%s - Too many arguments. Try `help %s extended'\n",
			cmdname, cmdname);
		return CMD_FAILURE;
	}

	buff = (char *) malloc(PATH_MAX);
	if (NULL == buff) {
		cli_error(CL_ENOMEM, "%s: ", cmdname);
		return CMD_FAILURE;
	}
	memset(buff, 0, sizeof(buff));

	if (argc == 1)
		getcwd(buff, PATH_MAX);
	else
		strncpy(buff, argv[1], PATH_MAX);

	scope = ls_scope(buff);

	switch (scope) {
	case LS_BOGUS:
		cli_error(CL_ENOENT, buff);
		free(buff);
		return CMD_FAILURE;
	case LS_FILE:
		ls_print(buff);
		break;
	case LS_DIR:
		dirp = opendir(buff);
		while ((dp = readdir(dirp)))
			ls_print(dp->d_name);
		closedir(dirp);
		break;
	}

	free(buff);

	return CMD_SUCCESS;
	
}

